package F5::ASMConfig::Handler::BotSignatureFile;

use strict;
use warnings;

use base qw/F5::ASMConfig::Handler::Base/;
use F5::DbUtils;

use F5::BotSignatures;
use F5::BotSignatureFile::Update;

our @handled_functions = (
                           {
                             function_name => 'update_bot_signature_file',
                             filename_arguments => ['3.filename'],
                             write_access => 1,
                             exclusive_lock_file => $F5::DbUtils::LOCK_FILE,
                           }
                         );

#--------------------------------------------------------------------
sub update_bot_signature_file {
    my($self, $session, $call_flags, $place_holder, $bot_signature_file) = @_;

    my %params = (
                  dbh => $self->{dbh},
                  filename => $bot_signature_file->{filename} || "",
                  username => $call_flags->{user_name} || "",
                  in_wrapping_transaction => $self->{in_transaction},
                  logger => $self->{logger},
                  outbound_call_timeout => $call_flags->{outbound_call_timeout},
                  originator => $call_flags->{originator},
                  asm_config_handler => $self,
                  skip_set_active => $call_flags->{skip_set_active},
                  skip_set_sigs_active => $call_flags->{skip_set_sigs_active},
                 );


    my $rv;
    eval {
        my $bot_signature_update = F5::BotSignatureFile::Update->new(%params);
        $rv = $bot_signature_update->update(%params);
    };

    if ($@) {
        my $orig_error = $@;
        my $error_thrown;
        if (ref($orig_error) && $orig_error->{error_msg}) {
            $error_thrown = $orig_error->{error_msg};
        }
        else {
            $error_thrown = "Bot Signature File update failed. Error: $@";
        }

        if (ref($error_thrown) && (exists $error_thrown->{error_msg})) {
            my $error_msg_secured = $error_thrown->{error_msg};
            # Error message may look like this: "User signature file does not conform to schema (/ts/var/negsig/admin~OhDF4F2qSU1rjaLPfh-8bA:4: parser error : Prema..."
            # We don't want the file location to be shown to user.
            $error_msg_secured =~ s/\([^\s]+\s/\(/;
            $rv = {completed_successfully => 0,
                   error_msg => $error_msg_secured};
        }
        else {
            return $self->handle_error(action => 'update_bot_signature_file',
                                       error_thrown => $error_thrown,
                                       error_level  => 'error',
                                      );
        }
    }

    return { return_value => F5::ASMConfig::BotSignatureFileUpdateRes->new(%{$rv})};

}

1;
