package F5::BehavioralDos;

use strict;
use warnings;

use F5::ASMConfig::Cache;
use F5::ASMConfig::Entity::BehavioralDos;
use F5::ASMConfig::Entity::DosProtection;

use YAML::XS;
use Google::ProtocolBuffers::Dynamic;

our $PROTO_PATH = F5::Deploy::map_path('/usr/share/ts/share/proto/');
our $BADOS_PROTO_FILENAME = 'bados.proto';
my $are_protobufs_loaded = 0;

#----------------------------------------------------------------------

sub new {
    my($class, %params) = @_;
    my $self = {};
    bless $self, $class;
    $self->init(%params);
    return $self;
}

#----------------------------------------------------------------------

sub init {
    my($self, %params) = @_;
    $self->{dbh} = $params{dbh};
    $self->{logger} = $params{logger};

    $self->prepare_protobuf_modules();
}

#---------------------------------------------------------

sub prepare_protobuf_modules {
    return 1 if $are_protobufs_loaded;

    my $protobuf = Google::ProtocolBuffers::Dynamic->new($PROTO_PATH);
    $protobuf->load_file($BADOS_PROTO_FILENAME);
    $protobuf->map({package => 'bados', prefix => "Bados",
                    options => { encode_defaults_proto3 => 1,
                                 explicit_defaults => 1,
                               }
                    });

    $are_protobufs_loaded = 1;
}

#---------------------------------------------------------

sub get_bados_config_message {
    my($self, %params) = @_;
    my $virtual_server_profiles = $params{virtual_server_profiles};
    my $transaction_id = $params{transaction_id};

    my $dbh = $self->{dbh};

    my @bados_vsp_elements;
    for my $vsp (@$virtual_server_profiles) {
        my $policy = F5::ASMConfig::Cache::get_cached_policy(dbh => $dbh,
                                                             policy => {policy_name => $vsp->{profile_name} },
                                                            );
        my $dos_protection = F5::ASMConfig::Entity::DosProtection->new(dbh => $dbh,
                                                                       policy => $policy,
                                                                       get_criteria => {},
                                                                      );
        if (! $dos_protection->{flg_enabled}) {
            next;
        }

        my $bados_config = F5::ASMConfig::Entity::BehavioralDos->new(dbh => $dbh,
                                                                     policy => $policy,
                                                                     get_criteria => {},
                                                                    );
        my $bados_vsp_element = Bados::VirtualServerProfile->new({
            vs_name => $vsp->{vs_name},
            profile_name => $vsp->{profile_name},
            mitigation_level => $bados_config->{mitigation_level},
            enable_http_signatures => $bados_config->{flg_http_signatures},
            enable_tls_signatures => $bados_config->{flg_tls_signatures},
            bad_actor_detection => Bados::VirtualServerProfile::BadActorDetection->new({
                is_enabled => $bados_config->{flg_bad_actor_detection},
                enable_tls_indexing => $bados_config->{flg_bad_actor_detection_tls_indexing},
            }),
        });
        push(@bados_vsp_elements, $bados_vsp_element);
    }
    my $bados_config_message = Bados::BadosConfig->new({
        transaction_id =>  $transaction_id,
        virtual_server_profile => \@bados_vsp_elements,
    });

    $self->{logger}->trace($bados_config_message) if $self->{logger};

    return Bados::BadosConfig->encode($bados_config_message);
}

#---------------------------------------------------------

sub deserialize_bados_config_message {
    my($self, %params) = @_;
    my $message = $params{message};

    return Bados::BadosConfigRequest->decode($message);
}

#---------------------------------------------------------

1;


