package F5::ClientSideJS;

use F5::DateTimeUtils;
use F5::Deploy;
use File::Copy;
use File::Path;
use F5::Utils::File;

our $BASE_CLIENTSIDE_DIR = F5::Deploy::get_path_by_type('clientside_js');
our $CONF_OUTPUT_TMP_DIR = $BASE_CLIENTSIDE_DIR . '/compiler_tmp';
our $CONF_OUTPUT_DIR = $BASE_CLIENTSIDE_DIR . '/trigger-queue';
our $OBFUSCATED_INPUT_DIR = $BASE_CLIENTSIDE_DIR . '/generated-queue';
our $CLIENT_SIDE_JS_DEST_DIR = F5::Deploy::get_path_by_type('bd_config_files') . '/client-side-js';

# Temporary setting the master key hardcoded (here and in bd). This will be solved in: https://jira.pdsea.f5net.com/browse/MBIPWAF-183225
our $MASTER_KEY = "62de1023fba31f763943ce1cecf86d64528b96417ca1663e4153ce21b067bfd9dfb52da6aa4d1c00f6caeba833a7ba66cc";


#----------------------------------------------------------------------

sub new {
    my($class, %params) = @_;
    my $self = {};
    bless $self, $class;
    $self->init(%params);
    return $self;
}

#----------------------------------------------------------------------

sub init {
    my($self, %params) = @_;
    $self->{logger} = $params{logger};
}

sub generate_obfuscation_config_file {
    my($self, %params) = @_;

    # First making sure required folders exists
    mkpath($CONF_OUTPUT_TMP_DIR);
    mkpath($CONF_OUTPUT_DIR);
    chmod(0777, $BASE_CLIENTSIDE_DIR);
    chmod(0777, $CONF_OUTPUT_DIR); 

    my $cur_time = time;
    my $time_string = F5::DateTimeUtils::convert_epoch_to_file_timestamp($cur_time);
    my $curr_filename = "obfuscator-$time_string.json";
    my $full_filename = "$CONF_OUTPUT_TMP_DIR/$curr_filename";

    my $rv = F5::SystemUtils::run_system_tool(command_line_params =>
                [ F5::Deploy::get_path_by_type('executables').'/datacrypt_cmd', '-g', $params{generated_variants_count},
                                                                                '-o'.$full_filename,
                                                                                '-a', $cur_time,
                                                                                '-m', $MASTER_KEY ]);

    if ($rv && !$rv->{completed_successfully}) {
        $self->{logger}->error("Failed to generate js obfuscation config file") if $self->{logger};
        #die $rv->{standard_error};
        return;
    }

    # clean all previous conf files from shared volume
    rmtree($CONF_OUTPUT_DIR);
    mkpath($CONF_OUTPUT_DIR);
    chmod(0777, $CONF_OUTPUT_DIR);

    # Copy created json file to shared volume
    move($full_filename, $CONF_OUTPUT_DIR);
    my $final_file_path = "$CONF_OUTPUT_DIR/$curr_filename";
    chmod(0777, $final_file_path);
 
    $self->{logger}->trace("Successfully generated js obfuscation config file in $final_file_path") if $self->{logger};
}

#----------------------------------------------------------------------

sub last_promoted_dir {
    my($self) = @_;

    my @dirs = sort glob("$CLIENT_SIDE_JS_DEST_DIR/*");
    if (@dirs) {
        return $dirs[-1];
    }
    return;
}

#----------------------------------------------------------------------

sub check_obfuscated_files {
    my($self) = @_;

    # Get the latest obfuscated folder
    my @folders = sort glob("$OBFUSCATED_INPUT_DIR/*");
    if (!@folders) {
        # Folder is empty, nothing to move
        return;
    }
    return 1;
}

1;
