package F5::PolicyValidator::AdVRHostSecret;

use strict;
use warnings;

use base 'F5::PolicyValidator::Default';
use F5::BigipUtils;

sub validate {
    my($self, %params) = @_;
    my $dbh = $params{dbh};
    my $logger = $params{logger};
    my $validator = $params{validator};
    my $policy = $params{policy};

    if (! F5::BigipUtils::is_feature_enabled('adaptive_violation_rating')) {
        return;
    }

    my $policy_advr_properties = $dbh->selectrow_hashref(<<EndQuery, { Slice => {} } , $policy->{policy_id});
SELECT flg_enabled
  FROM PLC.PL_ADAPTIVE_VIOLATION_RATING
 WHERE policy_id = ?
EndQuery

    if ($policy_advr_properties->{flg_enabled} ) {
        my $advr_properties = $dbh->selectrow_hashref(<<EndQuery, { Slice => {} });
SELECT host, secret
  FROM PLC.ADAPTIVE_VIOLATION_RATING
EndQuery

        if ( ($advr_properties->{host} =~ /^\s*$/ ) || ($advr_properties->{secret} =~ /^\s*$/ )) {
          $validator->report_error(
              error_type => $F5::PolicyValidator::ERROR_TYPE_LOGIC,
              error_msg  => 'Adaptive Violation Rating is enabled for this policy, but the training service is not configured',
              entity     => ''
              );
        }
    }
}

1;
