package F5::ProtocolBuffers;

use strict;
use warnings;

use Google::ProtocolBuffers::Dynamic;
use File::Find;
use File::Basename;
use File::Spec;
use File::Copy;
use File::Path;

#----------------------------------------------------------------------

sub init {
    my(%params) = @_;
    my $base_dir = $params{base_dir};
    my $package = $params{package};
    my $imports_dir = $params{imports_dir};

    my $was_protobuf_file_found = 0;

    return $was_protobuf_file_found unless -d $imports_dir;

    # copy all protobufs from the source dir to a local dir
    find({
          no_chdir => 1,
          follow_fast => 1,
          wanted => sub {
              my $filename = basename($File::Find::name);
              if (-f $File::Find::name && $filename =~ /\.proto$/) {
                  my $relative_path = File::Spec->abs2rel($File::Find::name, $base_dir);
                  my $relative_dir = dirname($relative_path);
                  my $destination_dir = $relative_dir eq "." ? $imports_dir : $imports_dir . $relative_dir;
                  mkpath($destination_dir) unless -d $destination_dir;
                  copy($File::Find::name, $destination_dir) unless -f "${destination_dir}/$filename";
              }
          },
         },
        $base_dir);

    my $p = Google::ProtocolBuffers::Dynamic->new($imports_dir);
    find({
          no_chdir => 1,
          wanted => sub {
              my $filename = basename($File::Find::name);
              if (-f $File::Find::name && $filename =~ /\.proto$/) {
                  my $relative_path = File::Spec->abs2rel($File::Find::name, $imports_dir);
                  $p->load_file($relative_path);
                  $was_protobuf_file_found = 1;
              }
          },
         },
         $imports_dir);

    if ($was_protobuf_file_found) {
        $p->map({package => "google.protobuf", prefix => "google.protobuf"},
                {package => $package, prefix => $package});
    }

    return $was_protobuf_file_found;
}

#----------------------------------------------------------------------

1;
