package F5::REST::WAF;

use strict;
use warnings;

use Mojolicious::Lite;
use F5::Logger;

use F5::REST::WAF::ConvertPolicy;

my $action_to_class_map = {
    'convert-policy' => 'F5::REST::WAF::ConvertPolicy',
};

#----------------------------------------------------------------------

sub new {
    my($class, %params) = @_;

    my $version = $params{version};
    my $rest_action = $params{rest_action};

    # currently support only version 'v1'
    if ($version ne 'v1') {
        die "Unsupported version: '$version'";
    }

    my $handler_class = $action_to_class_map->{$rest_action};
    if (! $handler_class) {
        die "Unsupported action: '$rest_action'";
    }

    my $self = {};
    bless $self, $handler_class;
    $self->init(%params);

    return $self;
}

#----------------------------------------------------------------------

sub init {
    my ($self, %params) = @_;

    $self->{logger} = $params{logger} || F5::Logger->new('/ts/log/waf_rest.log');
    $self->{connection} = $params{connection};
}

#----------------------------------------------------------------------

sub process {
    # abstract function, implementation in action subclass
}

#----------------------------------------------------------------------


1;

